// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SceneItemGenerator.hpp"

using namespace Dawn;

void SceneItemGenerator::generate(
  int32_t &assetNumber,
  int32_t &componentNumber,
  int32_t &childNumber,
  std::map<std::string, std::string> &assetMap,
  std::vector<std::string> &includes,
  std::vector<std::string> *publicProperties,
  std::vector<std::string> *initBody,
  std::vector<std::string> *assetBody,
  std::string parentRef,
  struct SceneItem *item,
  std::string tabs
) {
  auto name = "itm" + std::to_string(childNumber++);

  if(item->ref == "this") {
    name = item->ref;
  } else {
    bool_t init = true;
    if(item->ref.size() > 0) {
      init = false;
      name = item->ref;
      if(item->ref != "this") {
        line(publicProperties, "SceneItemComponent *" + name + " = nullptr;", "");
      }
    }
    line(initBody, (init ? "auto " : "") + name + " = scene->createSceneItem();", "");
  }

  // Process extra properties
  if(item->position.size() > 0) {
    line(initBody, name + "->transform.setLocalPosition(" + item->position + ");", "");
  }

  if(item->scale.size() > 0) {
    line(initBody, name + "->transform.setLocalScale(" + item->scale + ");", "");
  }

  // Add assets
  auto itAssets = item->assets.begin();
  while(itAssets != item->assets.end()) {
    SceneAssetGenerator::generate(
      assetMap,
      assetNumber,
      initBody,
      assetBody,
      &(*itAssets),
      ""
    );
    ++itAssets;
  }

  // Add components for children
  auto itComponents = item->components.begin();
  while(itComponents != item->components.end()) {
    SceneItemComponentGenerator::generate(
      assetMap,
      componentNumber,
      includes,
      name,
      publicProperties,
      initBody,
      &(*itComponents),
      ""
    );
    ++itComponents;
  }

  // Process sub children
  auto itChildren = item->children.begin();
  while(itChildren != item->children.end()) {
    SceneItemGenerator::generate(
      assetNumber,
      componentNumber,
      childNumber,
      assetMap,
      includes,
      publicProperties,
      initBody,
      assetBody,
      name,
      &(*itChildren),
      ""
    );
    ++itChildren;
  }

  // Set parent
  if(!parentRef.empty()) {
    line(initBody, name + "->transform.setParent(&"+parentRef+"->transform);", "");
  }
}